package com.sensorsdata.analytics.android.sdk.session;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.Application;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;

import com.sensorsdata.analytics.android.sdk.SensorsDataAPI;

import java.lang.ref.WeakReference;

/**
 * Created by HPY.
 * Time: 2018/8/28.
 * Desc: 监听前后台切换，对会话动态初始化，目前切换后台一分钟后失效
 */

@SuppressLint("NewApi")
public enum SessionMonitor implements Application.ActivityLifecycleCallbacks {
    INSTANCE;
    private final static String TAG = "SessionMonitor";

    private final static int CHECK_DELAY = 800;

    private boolean foreground;
    private boolean paused;

    private final Handler handler = new Handler(Looper.getMainLooper());
    private WeakReference<Activity> topActivityRef = new WeakReference<>(null);
//    private int count = 0;

    @Override
    public void onActivityStopped(final Activity activity) {
//        count--;
//        if (count == 0) {
//            //后台
//            Log.i("SessionMonitor", "onActivityStopped: ");
//            SensorsDataAPI.sharedInstance(activity).setSessionTimestamp(System.currentTimeMillis());
//        }

        Activity topActivity = topActivityRef.get();
        if (topActivity == activity) {
            paused = true;
            handler.removeCallbacksAndMessages(null);
            handler.postDelayed(new Runnable() {
                @Override
                public void run() {
                    if (foreground && paused) {
                        foreground = false;
//                        Log.i(TAG, "went background");
                        SensorsDataAPI.sharedInstance(activity).setSessionTimestamp(System.currentTimeMillis());
                    }
                }
            }, CHECK_DELAY);
        }
    }

    @Override
    public void onActivityStarted(Activity activity) {
//        if (count == 0) {
//            //前台
//            Log.i("SessionMonitor", "onActivityStarted: ");
//            SensorsDataAPI.sharedInstance(activity).initSessionID();
//        }
//        count++;
    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle outState) {
    }

    @Override
    public void onActivityResumed(Activity activity) {
        handler.removeCallbacksAndMessages(null);
        topActivityRef = new WeakReference<>(activity);
        paused = false;

        boolean wasBackground = !foreground;
        foreground = true;

        if (wasBackground) {
//            Log.i(TAG, "went foreground");
            SensorsDataAPI.sharedInstance(activity).initSessionID();
        }
    }

    @Override
    public void onActivityPaused(Activity activity) {
    }

    @Override
    public void onActivityDestroyed(Activity activity) {
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle savedInstanceState) {
    }

    public static SessionMonitor getInstance() {
        return INSTANCE;
    }

    /**
     * 必须在初始化 SensorSDK 之后才能注册调用。
     *
     * @param app
     */
    public void bindApplication(Application app) {
        app.registerActivityLifecycleCallbacks(this);
    }

}
